"""
A simple pytest plugin which collects test information and prints it at the end
of the tests
"""
from collections import defaultdict
import pytest

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions


class InfoCollector(object):
    """
    Main driver of test information collection
    """
    def __init__(self):
        self._info = defaultdict(list)

    def pytest_terminal_summary(self, terminalreporter, exitstatus):
        """
        Hook for printing test info at the end of the run
        """
        # pylint: disable=unused-argument
        terminalreporter.section("Test Information")
        for test, info in self._info.items():
            for datum in info:
                terminalreporter.write("{}: {}\n".format(test, datum))

    @pytest.fixture
    def test_info(self, request):
        """
        Fixture to collect test information
        """
        def add_info(info):
            """
            Adds information about test
            """
            self._info[get_test_name(request)].append(info)
        return add_info


def get_test_name(request):
    """
    Get the name of test from pytest
    """
    return request.node.name


def pytest_configure(config):
    """Add plugin to pytest"""
    config.pluginmanager.register(InfoCollector(), "info_collector")



pytest_plugins = "pytester"  # to get testdir fixture


def test_single(testdir):
    testdir.makepyfile("""
        def test_single(test_info):
            test_info("A simple test")
    """)
    result = testdir.runpytest()
    result.stdout.fnmatch_lines("""
        *test_single: A simple test
    """)


def test_multiple(testdir):
    testdir.makepyfile("""
        def test_multiple(test_info):
            test_info("First one")
            test_info("Second one")
    """)
    result = testdir.runpytest()
    result.stdout.fnmatch_lines("""
        test_multiple: First one
        test_multiple: Second one
    """)

def test_many_tests(testdir):
    testdir.makepyfile("""
        def test_many_tests1(test_info):
            test_info("First one")
            test_info("Second one")

        def test_many_tests2(test_info):
            test_info("Second test one")
            test_info("Second test two")
    """)
    result = testdir.runpytest()
    result.stdout.fnmatch_lines("""
        test_many_tests1: First one
        test_many_tests1: Second one
    """)
    result.stdout.fnmatch_lines("""
        test_many_tests2: Second test one
        test_many_tests2: Second test two
    """)




from setuptools import setup
import versioneer

DESCRIPTION_FILES = ["pypi-intro.rst"]

long_description = []
import codecs
for filename in DESCRIPTION_FILES:
    with codecs.open(filename, 'r', 'utf-8') as f:
        long_description.append(f.read())
long_description = "\n".join(long_description)

setup(
    name="pytest-info-collector",
    version=versioneer.get_version(),
    packages = ['pytest_info_collector'],
    install_requires = ["pytest"],
    author = "James Tocknell",
    author_email = "aragilar@gmail.com",
    description = "pytest plugin to collect information from tests",
    long_description = long_description,
    license = "3-clause BSD",
    keywords = "pytest testing",
    url = "https://pytest-info-collector.readthedocs.io",
    entry_points = {
        'pytest11': [
            'name_of_plugin = pytest_info_collector',
        ]
    },
    classifiers=[
        'Framework :: Pytest',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    cmdclass=versioneer.get_cmdclass(),
)
